/* Parser for Nifty50 by Aamir Jahan, COMS W4115, Fall 2014 */

%{ open Ast %}

/* tokens expressed in nifty50, same as the ones in scanner */
%token SEMI LPAREN RPAREN LBRACE RBRACE LBRACKET RBRACKET COMMA METHOD
%token PLUS MINUS TIMES DIVIDE ASSIGN
%token EQ NEQ LT LEQ GT GEQ
%token AND OR XOR NAND NOR NOT BSL BSR
%token RETURN IF ELSE FOR WHILE 
%token <int> ILITERAL
%token <float> FLITERAL
%token <string> SLITERAL
%token <string> ID
%token EOF

/* other operators */
%token ADDRESS SWAP SOURCE MAP
%token INT DOUBLE PORT STRING
%token ENUM STRING 
%token INTERRUPT TYPE METHOD VOID
%token TRUE FALSE

/* associativity rules, priority increases downwards */
%nonassoc NOELSE
%nonassoc ELSE
%right ASSIGN
%left NOT
%left EQ NEQ
%left ADDRESS
%left SWAP
%left COMMA
%left LT GT LEQ GEQ
%left AND OR XOR NAND NOR
%left BSL BSR
%left PLUS MINUS
%left TIMES DIVIDE

%start program
%type <Ast.program> program

%%


/* Grammar portion */

program:
   /* nothing */ { [], [], [], []}
 | program vdecl { List.rev ($2 :: first $1), second $1, third $1, fourth $1 }
 | program fdecl { first $1, List.rev ($2 :: second $1), third $1, fourth $1 }
 | program idecl { first $1, second $1, List.rev ($2 :: third $1), fourth $1 }
 | program ydecl { first $1, second $1, third $1, List.rev ($2 :: fourth $1) }

vdecl:
   return_type ID SEMI 
   {  
    {
      vtype = $1;
      vname = $2; 
    };
   }
    | return_type ID array_id SEMI 
     { 
      {
        vtype = Array($1, $3);
        vname = $2; 
      }
    }  
		
fdecl:
   return_type ID LPAREN formals_opt RPAREN LBRACE vdecl_list stmt_list_opt RBRACE
   { 
    { 
      rtype = $1; 
      fname = $2;
      formals = $4;
      locals = List.rev $7;
      body = List.rev $8;
      } 
    }
		
idecl:
  INTERRUPT ID LBRACE vdecl_list stmt_list_opt RBRACE 
  { 
    {
      iname = $2;
      ilocals = List.rev $4;
      ibody = List.rev $5;
    }
  }

ydecl:
  TYPE ID LBRACE vdecl_list fdecl_opt RBRACE 
  { 
    {
      ytype = NewType($2);
      yname = $2;
      yproperties = $4;
      yfunctions = $5; 
    }
  }

/*describing formals list */
formals_opt:
    /* nothing */ { [] }
  | formal_list   { List.rev $1 }

formal_list:
  formal /* nothing */        { [$1] }  
  | formal_list COMMA formal  { $3 :: $1 }

formal:
  return_type ID
  {
    {
      vtype = $1;
      vname = $2;    
    }
  }
  | return_type ID array_id 
  {
    {
      vtype = Array($1, $3);
      vname = $2;
    }
  }

/* describing variable decl list */
vdecl_list:
    /* nothing */    { [] }
  | vdecl_list vdecl { $2 :: $1 }
 
/* describing array initialization*/
array_id:
  LBRACKET ILITERAL RBRACKET { string_of_int $2 }  
  | LBRACKET ID RBRACKET { $2 }  

array_size:
  ILITERAL { Iliteral($1) }
  | ID { Id($1) }

/* describing function declaration */
fdecl_list:
  | fdecl       { [$1] }
  | fdecl_list fdecl { $2 :: $1 }

fdecl_opt:
    /* nothing */ { [] }
  | fdecl_list   { List.rev $1 }

/*describing all possible return types */
return_type:
   VOID         {Void}
  | PORT        {Port}
  | INT         {Int}
  | STRING      {String}
  | DOUBLE      {Double}
  | ID          {NewType($1)}

/* describing statements */
stmt_list_opt:
      /* nothing */  { [] }
  | stmt_list { List.rev $1 }

stmt_list:
   stmt /* nothing */  { [$1] }
  | stmt_list stmt { $2 :: $1 }

stmt:
    expr SEMI { Expr($1) }
  | RETURN expr SEMI { Return($2) }
  | LBRACE stmt_list RBRACE { Block(List.rev $2) }
  | IF LPAREN expr RPAREN stmt %prec NOELSE { If($3, $5, Block([])) }
  | IF LPAREN expr RPAREN stmt ELSE stmt    { If($3, $5, $7) }
  | FOR LPAREN expr_opt SEMI expr_opt SEMI expr_opt RPAREN LBRACE stmt_list_opt RBRACE
     { For($3, $5, $7, $10) }
  | WHILE LPAREN expr RPAREN LBRACE stmt_list_opt RBRACE { While($3, $6) }


expr_opt:
    /* nothing */ { Noexpr }
  | expr          { $1 }

expr:
  | ILITERAL          	{ Iliteral($1) }
	| FLITERAL						{ Fliteral($1) }
	| SLITERAL						{ Sliteral($1) }
  | ID                	{ Id($1)}
  | ID array_id      		{ArrayIndex($1, $2) }
  | ID  METHOD ID array_id { METHOD(Id($1), (ArrayIndex($3, $4))) }
  | NOT expr          { Unop(Not, $2) }			/*Test this line*/
  | expr PLUS   expr  { Binop($1, Add,   $3) }
  | expr MINUS  expr  { Binop($1, Sub,   $3) }
  | expr TIMES  expr  { Binop($1, Mult,  $3) }
  | expr DIVIDE expr  { Binop($1, Div,   $3) }
  | expr EQ     expr  { Binop($1, Equal, $3) }
  | expr NEQ    expr  { Binop($1, Neq,   $3) }
  | expr LT     expr  { Binop($1, Less,  $3) }
  | expr LEQ    expr  { Binop($1, Leq,   $3) }
  | expr GT     expr  { Binop($1, Greater,  $3) }
  | expr GEQ    expr  { Binop($1, Geq,   $3) }
  | expr OR    expr   { Binop($1, Or,   $3) }
  | expr AND   expr   { Binop($1, And,   $3) }  
	| expr NOR    expr  { Binop($1, Nor,   $3) }
  | expr NAND   expr  { Binop($1, Nand,   $3) }
  | expr BSR    expr  { Binop($1, Bsr,   $3) }  
  | expr BSL    expr  { Binop($1, Bsl,   $3) }  
  | expr XOR    expr  { Binop($1, Xor,   $3) }  
  | expr ASSIGN expr  { Assign($1, $3) }
  | ID ADDRESS        { GetAddress($1)}
  | ID METHOD SOURCE ASSIGN expr  { Signal($1, $5) }
  | ID METHOD ADDRESS ASSIGN expr { Address($1, $5)}
  | ID METHOD ADDRESS ASSIGN ID METHOD ADDRESS { Address($1, Id($5))}
  | ID METHOD SWAP LPAREN RPAREN {Swap($1)}
  | ID LPAREN actuals_opt RPAREN { Call($1, $3) }
  | ID METHOD ID LPAREN actuals_opt RPAREN { MethodCall($1, $3, $5) }
  | ID METHOD MAP LPAREN ID COMMA expr COMMA expr RPAREN { Map($1, $5, $7, $9)}
  | LPAREN expr RPAREN { $2 }

actuals_opt:
    /* nothing */ { [] }
  | actuals_list  { List.rev $1 }

actuals_list:
    expr                    { [$1] }
  | actuals_list COMMA expr { $3 :: $1 }