(* Scanner for Nifty50 by Aamir Jahan, COMS W4115, Fall 2014 *)

{ open Parser LineCount}

(* declare digits, exponent, lxms, doubles, and strings *)
let digits = ['0'-'9']+
let exp = 'e'('+'|'-')? digits
let lxm = ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']*
let double = (digits exp? | digits '.' digits? exp? | digits '.' exp)
let str = '"' [^'"']* '"'


rule token = parse
	['\t' ' ' '\n' '\r' ] {token lexbuf} (*these tokens are considered as whitespace*)
| "/*"	{comment lexbuf}							(*denotes start of a comment*)
(* Groupings, Separators, Punctuators *)
| '('		{ LPAREN }
| ')'		{ RPAREN }
| '{'		{ LBRACE }
| '}'		{ RBRACE }
| ';' 	{ SEMI }
| ','   { COMMA }
| '[' 	{ LBRACKET }
| ']'		{ RBRACKET }
| '.'		{ METHOD }
(* Operators, Binary and Boolean *)
| '+'   { PLUS }
| '-'   { MINUS }
| '*'   { TIMES }
| '/'   { DIVIDE }
| '='   { ASSIGN }
| "=="  { EQ }
| "!="  { NEQ }
| '<'   { LT }
| "<="  { LEQ }
| ">"   { GT }
| ">="  { GEQ }  
| "<<"				{ BSL }
| ">>"				{ BSR }
| "and"       { AND }
| "or"        { OR }
| "xor"       { XOR }
| "nand"      { NAND }
| "nor"       { NOR }
| "not"       { NOT }
(* Conditional Statements, Loops *)
| "return" 		{ RETURN }
| "if"     		{ IF }
| "else"   		{ ELSE }
| "for"    		{ FOR }
| "while"  		{ WHILE }
(* data types *)
| "int"    		{ INT }
| "double"		{ DOUBLE }
| "string"		{ STRING }
| "port"			{ PORT }
| "void"			{ VOID }
| "True"			{ TRUE }
| "False"			{ FALSE }
(* Special Functions *)
| "Source"		{ SOURCE }
| "Map"				{ MAP }
| "Interrupt"	{ INTERRUPT }
| "Enum"			{ ENUM }
| "Address"		{ ADDRESS }
| "Swap"			{ SWAP }
| "Type"			{ TYPE }
(* digits, identifiers *)
| digits as integer { ILITERAL(int_of_string integer) }
| double as dble { FLITERAL(float_of_string dble) }
| lxm as id 	{ID(id) }
| str as slit {SLITERAL(slit)}
(* EOF functions *)
| eof { EOF }
| _ as char { raise (Failure("Illegal Character " ^ Char.escaped char)) }

and comment = parse
  "*/" { token lexbuf }
| _    { comment lexbuf }

(* End of File *)
